/*
 * sound.c
 *
 * (C)2000 Dan Potter
 *
 * These are some simple sound interface routines. Most of the bulk of the work
 * is done inside s3mplay, and this part just loads things into the sound RAM and
 * gives our program inside the AICA instructions. 
 *
 * For copying information, please see main.c
 */

#include "dream.h"


/* AICA firmware bytes */
#include "s3mplay.h"


/* SMP interface, right below S3M address */
volatile unsigned long *snd_iface = (unsigned long*)0xa080ffc0;
#define SND_STATUS	0
#define SND_MONO	1
#define SND_SMP		2
#define SND_SMP_WHERE	3
#define SND_SMP_SIZE	4
#define SND_SMP_HZ	5

/* Load a bit of data off the disk into sound RAM */
int snd_load(char *fn, int* idx) {
	unsigned char buffer[2048];
	int fd, r, oidx = *idx;

	dc_serial_printf("Loading %s at %08x\r\n", fn, idx);

	fd = dc_gd_open(fn, O_RDONLY);
	if (fd < 0) {
		dc_serial_printf("Can't open file %s\r\n", fn);
		return;
	}
	
	/* Load 2048 bytes at a time */
	while ( (r=dc_gd_read(fd, buffer, 2048)) > 0) {
		dc_snd_load(buffer, *idx, r);
		*idx += r;

	}
	dc_gd_close(fd);

	/* Align on 32-byte boundaries for the AICA chip */
	*idx += 32;
	if (*idx & 0x1f) {
		*idx += 32 - (*idx & 0x1f);
	}
	
	return *idx - oidx;
}

/* A list of the sound effects to load */
char* sfxlist[] = {
	"/AUDIO/ACTIVATE.RAW",
	"/AUDIO/CLICKED.RAW",
	"/AUDIO/TOGGLED.RAW",
	"/AUDIO/WNDCLOSE.RAW",
	"/AUDIO/DESKTOP.RAW",
	"/AUDIO/ICONBOX2.RAW"
};
int sfxaddrs[6] = {0};
int sfxsizes[6] = {0};

/* Loads a music file off the CD-ROM and starts it playing */
void sound_start(char *fn, int mono) {
	int idx, i;

	/* Re-initialize the sound */
	dc_snd_init();

	/* Load the S3M first at 0x10000 */
	idx = 0x10000;
	snd_load(fn, &idx);

	/* Load the sound effects after that */
	for (i=0; i<6; i++) {
		dc_serial_printf("Loading effect %s at %08x\r\n", sfxlist[i], idx);
		sfxaddrs[i] = idx;
		sfxsizes[i] = snd_load(sfxlist[i], &idx);
	}

	/* Check mono mode.. if turned on, tell S3MPlay */
	snd_iface[SND_MONO] = mono;

	/* Start the music playing */
	dc_serial_printf("Load OK, starting ARM\r\n");
	dc_snd_load_arm(s3mplay, sizeof(s3mplay));

	/* Wait for program to start (load cycle) */
	while (snd_iface[SND_STATUS] != 3)
		;

	/* Wait for program to play (end load cycle) */
	while (snd_iface[SND_STATUS] == 3)
		;
}

/* Play a single sound effect */
void sound_effect(int idx) {
	snd_iface[SND_SMP_WHERE] = sfxaddrs[idx];
	snd_iface[SND_SMP_SIZE] = sfxsizes[idx]/2;
	/* Max sample size is 65536, but we have to cut back 32 */
	if (snd_iface[SND_SMP_SIZE] > 65504)
		snd_iface[SND_SMP_SIZE] = 65504;
	snd_iface[SND_SMP_HZ] = 44100;
	snd_iface[SND_SMP] = 1;
}


/* Initialize the sound system and start the music */
int sound_mono = 0;
char sound_music_fn[] = "/AUDIO/BUD.S3M";
void sound_init() {
	sound_start(sound_music_fn, sound_mono);
}

/* Set the mode (mono/stereo) */
void sound_set_mode(int m) {
	sound_mono = m;
	snd_iface[SND_MONO] = m;
}

